'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var createShadowDOM = function (currentChild, setShadowItemFn) {
    var _a;
    if (!currentChild)
        return;
    var children = currentChild.props.children;
    if (Array.isArray(children)) {
        React__default.Children.forEach(children, function (child) {
            if (typeof child === "object")
                createShadowDOM(child, setShadowItemFn);
            else if (child.toString().trim().length) {
                setNewChild();
            }
        });
    }
    else if (typeof children === "object") {
        createShadowDOM(children, setShadowItemFn);
    }
    else if ((_a = children) === null || _a === void 0 ? void 0 : _a.toString().trim().length) {
        setNewChild();
    }
    function setNewChild() {
        setShadowItemFn(function (child) {
            if (child)
                return __spreadArrays(child, [currentChild]);
            return [currentChild];
        });
    }
};

var defaults = {
    TIME_TO_HIGHLIGHT: 0,
    TIME_TO_STOP_HIGHLIGHT: 2000,
    HIGHLIGHT_CLASS: "rc-highlight",
    HIGHLIGHT_UNIQUE_ID: "rc-highlight-id"
};

var setHighlight = function (child, id, newlyAddedOnly, isInitial, showAfter, hideAfter, highlightStyle, clearHighlightRef, setClearHighlightFn, updateClearHandler, uniqueId) {
    var _a;
    if (!child.ref)
        return;
    var element = child.ref.current;
    if ((newlyAddedOnly ? isInitial : !isInitial) &&
        ((_a = element) === null || _a === void 0 ? void 0 : _a.getAttribute(defaults.HIGHLIGHT_UNIQUE_ID)) === uniqueId) {
        if (clearHighlightRef) {
            clearTimeout(clearHighlightRef);
        }
        setTimeout(function () {
            element.className += " " + highlightStyle;
        }, showAfter);
        var clearHighlight = setTimeout(function () {
            element.className = element.className.substr(0, element.className.indexOf(highlightStyle) - 1);
        }, showAfter + hideAfter);
        setClearHighlightFn(clearHighlight);
        updateClearHandler(id, clearHighlight);
    }
};

var ShadowChild = function (_a) {
    var child = _a.child, id = _a.id, showAfter = _a.showAfter, hideAfter = _a.hideAfter, highlightClassName = _a.highlightClassName, clearHandler = _a.clearHandler, newlyAddedOnly = _a.newlyAddedOnly, updateClearHandler = _a.updateClearHandler, uniqueId = _a.uniqueId;
    var initialMount = React.useRef(true);
    var _b = React.useState(clearHandler), clearHighlightRef = _b[0], setClearHighlightRef = _b[1];
    var changedChildren = Array.isArray(child.props.children)
        ? child.props.children.find(function (c) { return c.toString().trim().length; })
        : child.props.children;
    if (child.ref) {
        var elementDOM = child.ref.current;
        elementDOM.setAttribute(defaults.HIGHLIGHT_UNIQUE_ID, uniqueId);
    }
    React.useEffect(function () {
        var _a;
        if (!newlyAddedOnly && ((_a = initialMount) === null || _a === void 0 ? void 0 : _a.current)) {
            initialMount.current = false;
            return;
        }
        setHighlight(child, id, newlyAddedOnly, initialMount.current, showAfter, hideAfter, highlightClassName, clearHighlightRef, setClearHighlightRef, updateClearHandler, uniqueId);
        initialMount.current = false;
    }, [changedChildren]);
    return React__default.createElement(React__default.Fragment, null);
};

/* eslint-disable no-console */
var addStyleString = function (str, isSSR) {
    if (isSSR === void 0) { isSSR = false; }
    if (!document || isSSR) {
        console.warn("\"react-change-highlight\", As you're using server-side rendering, it's a must to create and pass your css class to the highlightStyle attribute for \"ChangeHighlight\" component.");
        return;
    }
    var node = document.createElement("style");
    node.innerHTML = str;
    document.body.appendChild(node);
};

var getUUID = function () {
    return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
        var randomNum = (Math.random() * 16) | 0;
        var finalChar = c === "x" ? randomNum : (randomNum & 0x3) | 0x8;
        return finalChar.toString(16);
    });
};

var listOfClearHighlightFunctions = [];
var updateClearHandler = function (childId, Fn) {
    listOfClearHighlightFunctions[childId] = Fn;
};
var ChangeHighlight = function (_a) {
    var children = _a.children, containerClassName = _a.containerClassName, _b = _a.showAfter, showAfter = _b === void 0 ? defaults.TIME_TO_HIGHLIGHT : _b, _c = _a.hideAfter, hideAfter = _c === void 0 ? defaults.TIME_TO_STOP_HIGHLIGHT : _c, _d = _a.highlightClassName, highlightClassName = _d === void 0 ? defaults.HIGHLIGHT_CLASS : _d, _e = _a.mode, mode = _e === void 0 ? "change" : _e, _f = _a.disabled, disabled = _f === void 0 ? false : _f, _g = _a.ssr, ssr = _g === void 0 ? false : _g;
    var _h = React.useState([]), shadowDOM = _h[0], setShadowDOM = _h[1];
    var isInitialMount = React.useRef(true);
    var _j = React.useState(""), uniqueId = _j[0], setUniqueId = _j[1];
    React.useEffect(function () {
        if (!disabled) {
            if (isInitialMount.current) {
                setUniqueId(getUUID());
                if (!!highlightClassName) {
                    addStyleString("\n          ." + defaults.HIGHLIGHT_CLASS + " {\n            background: #f8ffb4 !important;\n            transition: 0.25s ease-in-out;\n          }\n        ", ssr);
                }
                isInitialMount.current = false;
            }
            setShadowDOM([]);
            createShadowDOM({ props: { children: children } }, setShadowDOM);
        }
    }, [children, disabled, ssr, highlightClassName]);
    return (React__default.createElement("div", { className: containerClassName },
        shadowDOM.map(function (child, index) { return (React__default.createElement(ShadowChild, { key: index, id: index, child: child, newlyAddedOnly: mode === "newOnly", showAfter: showAfter, hideAfter: hideAfter, highlightClassName: highlightClassName, clearHandler: listOfClearHighlightFunctions[index], updateClearHandler: updateClearHandler, uniqueId: uniqueId })); }),
        children));
};

module.exports = ChangeHighlight;
